#!/usr/bin/env python
""" Removes monomorphic SNPs from map and dat-files 

Author: Philipp Bayer, 2013"""

VERSION=1.1 # fixed minor problem with SNPs at end of line having an extra tab

import sys

if len(sys.argv) != 3:
    sys.stderr.write("Usage: %s map-file dat-file\n" %sys.argv[0])
    sys.exit(1)

assert ".map" in sys.argv[1]
map_dict = {}
map_fh = open(sys.argv[1])
map_fh_header = map_fh.readline()
if map_fh_header != "# fjFile = MAP\n":
    sys.stderr.write("Error: Map-file isn't a map-file\n")
    sys.exit(1)

list_of_snps = []
print "Parsing map"
for line in map_fh:
    ll = line.split("\t")
    name = ll[0]
    map_dict[name] = line
    list_of_snps.append(name)

dat_fh = open(sys.argv[2])
dat_header = dat_fh.readline()
snp_dict = {}
snp_names = dat_fh.readline().rstrip().split("\t")
for snp in snp_names[1:]:
    snp_dict[snp] = []

individuals = []
print "Parsing dat"
for line in dat_fh:
    ll = line.replace("\n","").split("\t")
    for allele, snp in zip(ll[1:], snp_names[1:]):
        snp_dict[snp].append(allele)
    individuals.append(ll[0])

from collections import Counter
monomorphic_snps = []
for snp in snp_dict:
    c = Counter(snp_dict[snp])
    del(c[""])
    del(c["E"])
    if len(c) == 1:
	print snp + " is monomorphic"
        monomorphic_snps.append(snp)

new_snps_header = [""]
for element in snp_names[1:]:
    if element in monomorphic_snps: 
        del(snp_dict[element])
	print "Deleting " + element
        continue
    new_snps_header.append(element)


print "Writing output"
out_map_fh = open(sys.argv[1].replace(".map", "_no_monomorphic.map"), "w")
if '.dat' in sys.argv[2]:
	out_dat_fh = open(sys.argv[2].replace(".dat","_no_monomorphic.dat"),"w")
else:
	out_dat_fh = open(sys.argv[2].replace(".genotype","_no_monomorphic.genotype"),"w")

out_dat_fh.write(dat_header)
out_dat_fh.write("\t".join(new_snps_header) + "\n")
counter = 0
for individual in individuals:
    ll = [individual]
    for snp in new_snps_header[1:]:
        ll.append( snp_dict[snp][counter])
    out_dat_fh.write("\t".join(ll)+"\n")
    counter += 1

out_map_fh.write(map_fh_header)
for snp in new_snps_header[1:]:
    out_map_fh.write(map_dict[snp])


