
GC_UPPER_LIMIT = 10000
GC_LOWER_LIMIT = 20
FORBIDDEN_THINGS = set(['', 'E', 'e', ' ', 'X', 'x'])

from argparse import ArgumentParser
import sys
import math

parser = ArgumentParser()
parser.add_argument('dat')
parser.add_argument('map')

args = parser.parse_args()

out_gc = open(args.dat.replace('.dat', '_gene_conversions.txt'), 'w')
out_co = open(args.dat.replace('.dat', '_crossovers.txt'), 'w')

sys.stderr.write('Writing to %s and %s\n' %(out_gc.name, out_co.name))
pos_dict = {} # key: SNP-name, value: position

#############################################################################

for line in open(args.map):
    if '#' in line:
        continue
    ll = line.rstrip('\n\r').replace(',','').split('\t')
    name, pos = ll[0], int(ll[2])
    pos_dict[name] = pos

#############################################################################

fh = open(args.dat)
fh.readline()
snp_names = fh.readline().rstrip('\n\r').split('\t')[1:]
# Count differences between outer_length and inner_length
co_diff = []
gc_diff = []
for line in fh:
    if '#' in line: continue
    ll = line.upper().rstrip('\n\r').split('\t')
    current_allele = ''
    changes = []
    outer_changes = []
    alleles = ll[1:]
    name = ll[0]
    all_cos = [] # we need to merge COs at the end
    out_gc.write(name + '\n')
    out_co.write(name + '\n')
    header = '#Allele\tStart SNP\tEnd SNP\tStart position\tEnd position\tLength\tOuter End SNP\tOuter End Position\tOuter End Length\n'
    out_gc.write(header)
    out_co.write(header)
    for index, allele in enumerate(alleles):
        if allele != '' and current_allele == '' and allele not in FORBIDDEN_THINGS:
            # initialize the first allele
            current_allele = allele
            start = index # start is not empty
        if current_allele != '' and allele != current_allele and allele not in FORBIDDEN_THINGS:
            # in this case, the allele before 'index' could be empty - go back until you find a non-empty allele
            end = index-1
            outer_end = end

            while alleles[end] == '':
                end -= 1
            if start != end:
                # we either have to extend the prior block, or we add a new one
                # is there even a prior block?
                if len(changes) == 0:
                    changes.append([current_allele, start, end])
                    outer_changes.append([current_allele, start, outer_end])

                else:
                    prior_block = changes[-1]
                    prior_outer_block = outer_changes[-1]
                    prior_allele = prior_block[0]
                    if prior_allele == current_allele:
                        # extend the prior block to this end
                        prior_block[2] = end
                        prior_outer_block[2] = outer_end
                        changes[-1] = prior_block
                    else:
                        # new block!
                        changes.append([current_allele, start, end])
                        outer_changes.append([current_allele, start, outer_end])
            current_allele = allele
            start = index
    last_pair = changes[-1]
    if last_pair[0] != start:
        # add one last pair if we haven't already
        end = len(alleles)-1
        outer_end = end
        while alleles[end] == '':
            end -= 1
        if start != end:
            changes.append([current_allele, start, end])
            outer_changes.append([current_allele, start, outer_end])
    list_of_cos = []
    index = 0

    current_block = ''
    for allele, start, end in changes:
        start_name, end_name = snp_names[start], snp_names[end]
        start_pos, end_pos = pos_dict[start_name], pos_dict[end_name]
        outer_end = outer_changes[index][-1]
        if outer_end == len(snp_names):
            outer_end -= 1
        outer_end_name = snp_names[outer_end]
        outer_end_pos = pos_dict[outer_end_name]
        length = end_pos - start_pos
        outer_length = outer_end_pos - start_pos

        diff = outer_length - length
        if diff != 0:
            diff = math.log(diff)

        if not current_block and (length >= GC_UPPER_LIMIT):
            current_block = [allele, start, end, outer_end]
            prev_allele = allele
            index += 1
            continue

        if current_block and (allele != current_block[0]) and (length >= GC_UPPER_LIMIT):
            # we have a new crossover, start a new block
            # first, write out the old block
            block_start, block_end, block_outer_end = current_block[1], current_block[2], current_block[3]
            block_start_name, block_end_name, block_outer_end_name = snp_names[block_start], snp_names[block_end], snp_names[block_outer_end]
            block_start_pos, block_end_pos, block_outer_end_pos = pos_dict[block_start_name], pos_dict[block_end_name], pos_dict[block_outer_end_name]
            block_length = block_end_pos - block_start_pos
            block_outer_length = block_outer_end_pos - block_start_pos
            assert length <= outer_length
            assert length > 0
            diff = block_outer_length - block_length

            co_diff.append(diff)
            line = '{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}\n'.format(current_block[0],  block_start_name, block_end_name, block_start_pos, block_end_pos, block_length, block_outer_end_name, block_outer_end_pos, block_outer_length)
            out_co.write(line)

            # now start the new block
            current_block = [allele, start, end, outer_end]
        elif current_block and (allele == current_block[0]) and (length >= GC_UPPER_LIMIT):
            # we have the same block, merge with current_block
            current_block[2] = end
            current_block[3] = outer_end
        elif GC_LOWER_LIMIT <= length < GC_UPPER_LIMIT:
            line = '{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\n'.format(allele,  start_name, end_name, start_pos, end_pos, length, outer_end_name, outer_end_pos, outer_length)
            # we have a gene conversion, write to GC file
            gc_diff.append(diff)
            out_gc.write(line)
        else:
            # We have a gene conversion, but too small
            assert length < GC_LOWER_LIMIT
        prev_allele = allele
        index += 1

    block_start, block_end, block_outer_end = current_block[1], current_block[2], current_block[3]
    block_start_name, block_end_name, block_outer_end_name = snp_names[block_start], snp_names[block_end], snp_names[block_outer_end]
    block_start_pos, block_end_pos, block_outer_end_pos = pos_dict[block_start_name], pos_dict[block_end_name], pos_dict[block_outer_end_name]
    block_length = block_end_pos - block_start_pos
    block_outer_length = block_outer_end_pos - block_start_pos
    assert length <= outer_length
    assert length > 0
    diff = block_outer_length - block_length

    co_diff.append(diff)
    line = '{0}\t{1}\t{2}\t{3}\t{4}\t{5}\t{6}\t{7}\t{8}\n'.format(current_block[0],  block_start_name, block_end_name, block_start_pos, block_end_pos, block_length, block_outer_end_name, block_outer_end_pos, block_outer_length)
    out_co.write(line)


'''
import matplotlib.pyplot as plt
plt.hist(gc_diff, alpha=0.5, bins=100, label='GC difference')
plt.hist(co_diff, alpha=0.5, bins=100, label='CO difference')
plt.legend()
plt.savefig('Outer_inner_end_differences.png')
plt.show()

import numpy
print 'GC Median, mean, min, max'
print numpy.median(gc_diff), numpy.mean(gc_diff), min(gc_diff), max(gc_diff)
print 'CO Median, mean, min, max'
print numpy.median(co_diff), numpy.mean(co_diff), min(co_diff), max(co_diff)
'''
