def parse_positions(fh):
    all_positions = [] # [start, end, outer_end]
    prev_end = ""
    maximum = 0
    for line in fh:
        ll = line.rstrip().split('\t')
        if len(ll) == 1:
            ind = ll[0]
            # skip the very first crossover block
            skip = True
            continue
        if line.startswith("#"):
            continue
        if skip == True:
            skip = False
            start, end, outer_end = int(ll[3]), int(ll[4]), int(ll[7])
            if end > maximum: maximum = end
            prev_end = int(ll[4])
            continue
        start, end, outer_end = int(ll[3]), int(ll[4]), int(ll[7])
        # prev_end is the outermost possible start
        # start is the innermost possible start
        # end is the innermost possible end
        # outer_end is the outermost possible end
        pos = [ind, prev_end, start, end, outer_end]
        if end > maximum: maximum = end

        all_positions.append( pos )
        prev_end = end

    print maximum
    return all_positions, str(maximum)

from argparse import ArgumentParser
import sys

parser = ArgumentParser()
parser.add_argument('table_to_filter', help='CO or GC table to filter')
parser.add_argument('type', help='GC or CO - adjacent COs are merged, adjacent GCs are not')

args = parser.parse_args()
fh = open(args.table_to_filter)
suffix = fh.name.split(".")
new_name = ".".join(suffix[:-1]) + "_filtered." + suffix[-1]
assert new_name != fh.name
#out = open(new_name, "w")

type = args.type
if (type != 'GC') and (type != 'CO'):
    sys.stderr.write("ERROR: --type must be GC or CO.\n")
    sys.exit(1)

all_positions, maximum = parse_positions(fh)

removed_counter = 0
skip_these = set() # Contains all positions that should be skipped in form [prev_end, start, end, outer_end)
for pos in all_positions:
    ind = pos[0]
    for other_pos in all_positions:
        other_ind = other_pos[0]
        if ind == other_ind:
            # don't compare individuals against themselves
            continue

        this_prev_end, this_start, this_end, this_outer_end = pos[1], pos[2], pos[3], pos[4]
        other_prev_end, other_start, other_end, other_outer_end = other_pos[1], other_pos[2], other_pos[3], other_pos[4]
        # now find out which pairs to skip
        remove = False

        # prev_end, start, end, outer_end
        # |    |--------|    |  ## other
        # |  |------------|  |  ## this
        if (this_start >= other_prev_end) and (this_start <= other_start) \
                and (this_end >= other_end) and (this_end <= other_outer_end):
            remove = True
        # now the opposite case
        # |  |-----------| | ## other
        # |    |-------|   | ## this
        elif (other_start >= this_prev_end) and (other_start <= this_start) \
                and (other_end >= this_end) and (other_end <= this_outer_end):
            remove = True
        # | |-------|   | ## other
        # |   |-------| | ## this
        elif (other_start >= this_prev_end) and (other_start <= this_start) \
                and (this_end >= other_end) and (this_end <= other_outer_end):
            remove = True
        # now the opposite again
        # |    |-------|  | ## other
        # |  |-------|    | ## this
        elif (this_start >= other_prev_end) and (this_start <= other_start) \
                and (other_end >= this_end) and (other_end <= this_outer_end):
            remove = True
        # now extremer: only start or end has to overlap!
        # only end:
        # ..---|  | ## other
        # .--|    | ## this
        elif (other_end >= this_end) and (other_end <= this_outer_end):
            remove = True
        # opposite:
        # ..-|    | ## other
        # .----|  | ## this
        elif (this_end >= other_end) and (this_end <= other_outer_end):
            remove = True
        # only start:
        #  |   |----. ## other
        #  |     |--. ## this
        elif (other_start >= this_prev_end) and (other_start <= this_start): # type == "CO"
            remove = True
        # opposite:
        #  |     |--. ## other
        #  |  |-----. ## this
        elif (other_start >= this_prev_end) and (other_start <= this_start): # type == "CO"
            remove = True

        if remove:
            removed_counter += 1

            print this_prev_end, this_start, this_end, this_outer_end
            print other_prev_end, other_start, other_end, other_outer_end
            skip_these.add( tuple([this_start, this_end]) )
            skip_these.add( tuple([other_start, other_end]) )

dada
sys.stderr.write("Removed {0} out of {1} positions\n".format(len(skip_these), len(all_positions)))
sys.stderr.write("In total {0} pairs overlapped\n".format(removed_counter))
# reset filehandle
fh.seek(0)
# now parse the file again, skip blocks, and merge adjacent crossovers if needed

start_line = False
current_block = ""
current_allele = ""
for line in fh:
    ll = line.rstrip("\n\r").split("\t")
    if line.startswith("#"):
        out.write(line)
        continue

    if len(ll) == 1:
        # line is ["TN100"]
        # do we have a block from the previous individual?
        if current_block:
            current_block[4], current_block[7] = maximum, maximum
            out.write("\t".join(current_block) + "\n")
        out.write(line)
        start_line = True
        ind = ll[0]
        continue

    if start_line and (type == "CO"):
        # next line after ["TN100"]
        # set current allele, set current block
        current_allele = ll[0]
        current_block = ll
        start_line = False
        continue

    # do we have to skip?
    start, end = map(int, ll[3:5])
    if (start, end) in skip_these:
        continue

    if type == "GC":
        # don't merge GCs
        out.write(line)
        continue

    this_allele = ll[0]
    # CO, we have to merge adjacent genotypes
    if this_allele == current_allele:
        # elongate the old block
        new_end, new_outer_end, new_end_snp, new_outer_end_snp = ll[4], ll[7], ll[2], ll[6]
        new_length = str(int(new_end) - int(current_block[3]))
        new_outer_length = str(int(new_outer_end) - int(current_block[3]))
        current_block[2] = new_end_snp
        current_block[4] = new_end
        current_block[5] = new_length
        current_block[6] = new_outer_end_snp
        current_block[7] = new_outer_end
        current_block[8] = new_outer_length
    else:
        # write out the old block
        out.write("\t".join(current_block) + "\n")
        # new block!
        current_block = ll
        current_allele = ll[0]

# have to print the last block
if type == "CO":
    current_block[4], current_block[7] = maximum, maximum
    out.write("\t".join(current_block) + "\n")
