import os, sys
from collections import OrderedDict
import math
from argparse import ArgumentParser
from numpy import arange # standard ranges don't support floats
import matplotlib.pyplot as plt
import matplotlib.ticker as tkr

CENTROMER_DICT = {
'A1': [15238707,16942790],
'A2':[8335693,9228343],
'A3':[31522604,31645090],
'A4':[6088010,6243256],
'A5':[12828548,14008325],
'A6':[11755719,12645967],
'A7':[4645131,5125645],
'A8':[5299827,5656133],
'A9':[10480631,11398296],
'A10':[6452226,7150219],
'C1':[18701169,18967279],
'C2':[33866933,36841765],
'C3':[44939471,46480712],
'C4':[18199394,18451727],
'C5':[26819882,27362221],
'C6':[28238786,29099493],
'C7':[8950963,9366604],
'C8':[6236267,6925072],
'C9':[24395747,25327920]}

def x_formatter(x, pos):
  return '{0:,}'.format(x/1000000.0)
  
x_format = tkr.FuncFormatter(x_formatter)

parser = ArgumentParser()
parser.add_argument('folder', help='Path to folder with txts containing GCs or COs')
parser.add_argument('type', help='CO or GC. Will choose different files from folder.')

args = parser.parse_args()
type = args.type

if type == 'GC':
  ending = 'gene_conversions_filtered.txt'
elif type == 'CO':
  ending = 'crossovers_filtered.txt'
else:
  sys.stderr.write('ERROR: Type must be GC or CO\n')
  sys.exit(1)

for element in os.listdir(args.folder):
  if element.endswith(ending):
    if ('C10' in element) or ('XA11' in element): continue
    fh = open(os.path.join(args.folder, element))
    chrom = element.split('_')[1]
    if 'X' in chrom: chrom = chrom.replace('X','').replace('m','')
    if 'A' in chrom and (chrom != 'A10'): chrom = chrom.replace('0','')
    try:
      centro = CENTROMER_DICT[chrom]
    except KeyError:
      centro = False
    ind_dict = {}
    biggest = 0 # need this to bin the chromosome later
    for line in fh:
      if line.startswith('#'): continue
      ll = line.rstrip().split('\t')
      if len(ll) == 1:
        ind = ll[0]
        ind_dict[ind] = []
      else:
	ind_dict[ind].append(ll)
        end = int(ll[7])
        if end > biggest: biggest = end
    if centro and biggest < centro[1]:
      biggest = centro[1] + 10000
    # now make a 100 bins, from 0 to the end ('biggest')
    stepsize = 0.01 * biggest
    bins = [float(i) for i in arange(0, biggest+stepsize, stepsize)]

    bin_dict = OrderedDict((number, 0) for number in bins)
    assert len(bin_dict) == 101
    #assert len(bins) == 101, len(bins)
    
    for ind in ind_dict:
      # don't count the initial haplotype with COs
      if type == 'CO': 
        recombinations = ind_dict[ind][1:]
      else:
        recombinations = ind_dict[ind]
      #recombinations = ind_dict[ind][1:]

      for r in recombinations:
        start = float(r[3])
        # find appropriate bin
        prev_b = 0
        found = False
        for b in bins:
          if start < b:
            bin_dict[prev_b] += 1
            found = True
            break
          prev_b = b
        if not found:
          last_bin = bin_dict.keys()[-1]
          bin_dict[last_bin] += 1

    x, y = bin_dict.keys(), bin_dict.values()
    new_x, new_y = [], []
    # get rid of 0s
    for a, b in zip(x,y):
      if b == 0: continue
      new_x.append(a)
      new_y.append(b)

    x, y = new_x, new_y
    plt.figure()
    ax = plt.subplot(111)
    ax.plot(x,y, 'k.')
    plt.ylim(0, max(y)+1)
    # Draw centromer lines
    if centro:
      ax.axvline(centro[0], ymin=0, ymax=max(y)+1,color='k')
      ax.axvline(centro[1], ymin=0, ymax=max(y)+1,color='k')
    #plt.title(element.split('_')[1] + '_filtered')
    plt.xlabel('Position on chromosome (in Gbp)')
    plt.ylabel('Number of crossovers in bin')
    plt.xlim(-1000, biggest+1000)
    ax.xaxis.set_major_formatter(x_format)
    if 'GBS' in element: # Brassica 
      plt.savefig('plots/%s.png' %element.split('_')[1], dpi=300, bbox_inches='tight')
    else: # Chickpea
      plt.savefig('plots/%s.png' %element.split('_')[0], dpi=300, bbox_inches='tight')
    #plt.show()
