def tn_repl(tn):
  return int(tn.replace('TN',''))
import os, sys
from collections import OrderedDict
import math
from argparse import ArgumentParser
from numpy import arange, mean, median # standard ranges don't support floats
import matplotlib.pyplot as plt

parser = ArgumentParser()
parser.add_argument('folder', help='Path to folder with txts containing GCs or COs')
parser.add_argument('type', help='CO or GC. Will choose different files from folder.')
parser.add_argument('-filter', help='Optional: Parse filtered table', action='store_true')

args = parser.parse_args()
type = args.type

if type == 'GC':
  ending = 'gene_conversions.txt'
elif type == 'CO':
  ending = 'crossovers.txt'
else:
  sys.stderr.write('ERROR: Type must be GC or CO\n')
  sys.exit(1)

if args.filter:
  ending = ending.replace('.txt','_filtered.txt')

ind_dict = {}
c_recs = []
a_recs = []

if args.filter:
  print 'Filtered'

print 'Chrom\t{0}s Min\tMax\tMean'.format(type)
c_ind_rec_dict = {}
a_ind_rec_dict = {}
for element in os.listdir(args.folder):
  if element.endswith(ending):
    if ('C10' in element) or ('XA11' in element): continue
    fh = open(os.path.join(args.folder, element))
    for line in fh:
      if line.startswith('#'): continue
      ll = line.rstrip().split('\t')
      if len(ll) == 1:
        ind = ll[0]
        ind_dict[ind] = []
      else:
        ind_dict[ind].append(ll)
  
    all_recs = []
    for ind in ind_dict:
      # have we counted recombinations for this one?
      if ind not in c_ind_rec_dict:
        c_ind_rec_dict[ind] = 0
        a_ind_rec_dict[ind] = 0
        
      if type == 'CO':
        recombinations = len(ind_dict[ind]) - 1
      else:
        recombinations = len(ind_dict[ind])

      all_recs.append(recombinations)
      if 'GBS_C' in element:
        c_recs.append(recombinations)
        c_ind_rec_dict[ind] += recombinations
      elif 'GBS_XA' in element:
        a_recs.append(recombinations)
        a_ind_rec_dict[ind] += recombinations
    print '{0}\t{1}\t{2}\t{3:.2f}'.format(element, min(all_recs), max(all_recs), mean(all_recs))

if c_recs:
  print 'C recombinations', sum(c_recs), 'AVG', sum(c_recs)/float(len(ind_dict))
  print 'A recombinations', sum(a_recs), 'AVG', sum(a_recs)/float(len(ind_dict))
print 'Individual\t{0}s A\t{0} C'.format(type)

for ind in sorted(a_ind_rec_dict, key=tn_repl):
  a_ind_recs = a_ind_rec_dict[ind]
  c_ind_recs = c_ind_rec_dict[ind]
  print '{0}\t{1}\t{2}'.format(ind, a_ind_recs, c_ind_recs)
if not c_recs:
  sys.exit(0)

from scipy.stats import ttest_ind
print 'T-test C and A'
print ttest_ind(c_recs, a_recs)
