import os, sys
from collections import OrderedDict
import math
from argparse import ArgumentParser
from numpy import arange # standard ranges don't support floats
import matplotlib.pyplot as plt

parser = ArgumentParser()
parser.add_argument('folder', help='Path to folder with txts containing GCs or COs')
parser.add_argument('type', help='CO or GC. Will choose different files from folder.')

args = parser.parse_args()
type = args.type

if type == 'GC':
  ending = 'gene_conversions.txt'
elif type == 'CO':
  ending = 'crossovers.txt'
else:
  sys.stderr.write('ERROR: Type must be GC or CO\n')
  sys.exit(1)
  


for element in os.listdir(args.folder):
  if element.endswith(ending):
    if ('C10' in element) or ('XA11' in element): continue

    bin_dict = OrderedDict((number, 0) for number in range(0,101))
    assert len(bin_dict) == 101

    fh = open(os.path.join(args.folder, element))
    ind_dict = {}
    biggest = 0 # need this to bin the chromosome later
    for line in fh:
      if line.startswith('#'): continue
      print line
      ll = line.rstrip().split('\t')
      if len(ll) == 1:
        ind = ll[0]
        ind_dict[ind] = []
      else:
	ind_dict[ind].append(ll)
        end = int(ll[7])
        if end > biggest: biggest = end
    # now make a 100 bins, from 0 to the end ('biggest')
    stepsize = 0.01 * biggest
    bins = [i for i in arange(0, biggest+stepsize, stepsize)]
    #assert len(bins) == 101, len(bins)
    
    for ind in ind_dict:
      # don't count the initial haplotype with COs
      if type == 'CO': 
        recombinations = ind_dict[ind][1:]
      else:
        recombinations = ind_dict[ind]
      #recombinations = ind_dict[ind][1:]

      for r in recombinations:
        start = float(r[3])
        # find appropriate bin
        prev_b = 0
        found = False
        for counter, b in enumerate(bins):
          if start < b:
            bin_dict[counter] += 1
            found = True
            break
          prev_b = b
        if not found:
          last_bin = bin_dict.keys()[-1]
          bin_dict[last_bin] += 1

    x, y = bin_dict.keys(), bin_dict.values()
    plt.figure()
    plt.plot(x,y, 'k')
    #plt.ylim(-1, max(y)+2)
    plt.xlim(-1, max(x)+2)
    plt.title(element.split('_')[1].replace('m','') + '_unfiltered')
    plt.savefig('plots/%s.png'%(element.split('_')[1].replace('m','') + '_unfiltered'))
    #plt.show()
